@@ -76,6 +76,7 @@ static pkgconf_client_t pkg_client;
 static const pkgconf_fragment_render_ops_t *want_render_ops = NULL;
 
 static uint64_t want_flags;
+static int verbosity = 0;
 static int maximum_traverse_depth = 2000;
 static size_t maximum_package_count = 0;
 
@@ -288,20 +289,33 @@ apply_digraph(pkgconf_client_t *client, pkgconf_pkg_t *world, void *unused, int
 #endif
 
 static bool
-apply_modversion(pkgconf_client_t *client, pkgconf_pkg_t *world, void *unused, int maxdepth)
+apply_modversion(pkgconf_client_t *client, pkgconf_pkg_t *world, void *data, int maxdepth)
 {
-	pkgconf_node_t *iter;
+	pkgconf_node_t *queue_iter;
+	pkgconf_list_t *pkgq = data;
 	(void) client;
-	(void) unused;
 	(void) maxdepth;
 
-	PKGCONF_FOREACH_LIST_ENTRY(world->required.head, iter)
+	PKGCONF_FOREACH_LIST_ENTRY(pkgq->head, queue_iter)
 	{
-		pkgconf_dependency_t *dep = iter->data;
-		pkgconf_pkg_t *pkg = dep->match;
+		pkgconf_node_t *world_iter;
+		pkgconf_queue_t *queue_node = queue_iter->data;
 
-		if (pkg->version != NULL)
-			printf("%s\n", pkg->version);
+		PKGCONF_FOREACH_LIST_ENTRY(world->required.head, world_iter)
+		{
+			pkgconf_dependency_t *dep = world_iter->data;
+			pkgconf_pkg_t *pkg = dep->match;
+
+			if (strncmp(pkg->id, queue_node->package, strlen(pkg->id)))
+				continue;
+
+			if (pkg->version != NULL) {
+				if (verbosity)
+					printf("%s: ", pkg->id);
+
+				printf("%s\n", pkg->version);
+			}
+		}
 	}
 
 	return true;
@@ -407,7 +421,8 @@ apply_env(pkgconf_client_t *client, pkgconf_pkg_t *world, void *env_prefix_p, in
 	char workbuf[PKGCONF_ITEM_SIZE];
 
 	for (it = want_env_prefix; *it != '\0'; it++)
-		if (!isalpha(*it) && !isdigit(*it))
+		if (!isalpha((unsigned char)*it) &&
+		    !isdigit((unsigned char)*it))
 			return false;
 
 	snprintf(workbuf, sizeof workbuf, "%s_CFLAGS", want_env_prefix);
@@ -642,6 +657,7 @@ usage(void)
 	printf("  --help                            this message\n");
 	printf("  --about                           print pkgconf version and license to stdout\n");
 	printf("  --version                         print supported pkg-config version to stdout\n");
+	printf("  --verbose                         print additional information\n");
 	printf("  --atleast-pkgconfig-version       check whether or not pkgconf is compatible\n");
 	printf("                                    with a specified pkg-config version\n");
 	printf("  --errors-to-stdout                print all errors on stdout instead of stderr\n");
@@ -908,6 +924,7 @@ main(int argc, char *argv[])
 		{ "personality", required_argument, NULL, 53 },
 #endif
 		{ "license", no_argument, &want_flags, PKG_DUMP_LICENSE },
+		{ "verbose", no_argument, NULL, 55 },
 		{ NULL, 0, NULL, 0 }
 	};
 
@@ -967,6 +984,9 @@ main(int argc, char *argv[])
 			personality = pkgconf_cross_personality_find(pkg_optarg);
 			break;
 #endif
+		case 55:
+			verbosity++;
+			break;
 		case '?':
 		case ':':
 			ret = EXIT_FAILURE;
@@ -1118,7 +1138,6 @@ main(int argc, char *argv[])
 		(want_flags & PKG_REQUIRES_PRIVATE) == PKG_REQUIRES_PRIVATE ||
 		(want_flags & PKG_PROVIDES) == PKG_PROVIDES ||
 		(want_flags & PKG_VARIABLES) == PKG_VARIABLES ||
-		(want_flags & PKG_MODVERSION) == PKG_MODVERSION ||
 		(want_flags & PKG_PATH) == PKG_PATH ||
 		want_variable != NULL))
 	{
@@ -1126,6 +1145,10 @@ main(int argc, char *argv[])
 		maximum_traverse_depth = 1;
 	}
 
+	/* we also want to walk only the flattened dependencies if we are requesting --modversion. */
+	if ((want_flags & PKG_MODVERSION) == PKG_MODVERSION)
+		maximum_traverse_depth = 1;
+
 	if (getenv("PKG_CONFIG_ALLOW_SYSTEM_CFLAGS") != NULL)
 		want_flags |= PKG_KEEP_SYSTEM_CFLAGS;
 
@@ -1324,10 +1347,10 @@ cleanup3:
 		/* check if there is a limit to the number of packages allowed to be included, if so and we have hit
 		 * the limit, stop adding packages to the queue.
 		 */
-		if (maximum_package_count > 0 && pkgq.length > maximum_package_count)
+		if (maximum_package_count > 0 && pkgq.length >= maximum_package_count)
 			break;
 
-		while (isspace((unsigned int)package[0]))
+		while (isspace((unsigned char)package[0]))
 			package++;
 
 		/* skip empty packages */
@@ -1382,41 +1405,41 @@ cleanup3:
 
 	if ((want_flags & PKG_DUMP_LICENSE) == PKG_DUMP_LICENSE)
 	{
-		apply_license(&pkg_client, &world, &ret, maximum_traverse_depth);
+		apply_license(&pkg_client, &world, &ret, 2);
 		goto out;
 	}
 
 	if ((want_flags & PKG_UNINSTALLED) == PKG_UNINSTALLED)
 	{
 		ret = EXIT_FAILURE;
-		apply_uninstalled(&pkg_client, &world, &ret, maximum_traverse_depth);
+		apply_uninstalled(&pkg_client, &world, &ret, 2);
 		goto out;
 	}
 
 	if (want_env_prefix != NULL)
 	{
-		apply_env(&pkg_client, &world, want_env_prefix, maximum_traverse_depth);
+		apply_env(&pkg_client, &world, want_env_prefix, 2);
 		want_flags = 0;
 	}
 
 	if ((want_flags & PKG_PROVIDES) == PKG_PROVIDES)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
-		apply_provides(&pkg_client, &world, NULL, maximum_traverse_depth);
+		apply_provides(&pkg_client, &world, NULL, 2);
 	}
 
 #ifndef PKGCONF_LITE
 	if ((want_flags & PKG_DIGRAPH) == PKG_DIGRAPH)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
-		apply_digraph(&pkg_client, &world, NULL, maximum_traverse_depth);
+		apply_digraph(&pkg_client, &world, NULL, 2);
 	}
 #endif
 
 	if ((want_flags & PKG_MODVERSION) == PKG_MODVERSION)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
-		apply_modversion(&pkg_client, &world, NULL, maximum_traverse_depth);
+		apply_modversion(&pkg_client, &world, &pkgq, 2);
 	}
 
 	if ((want_flags & PKG_PATH) == PKG_PATH)
@@ -1424,13 +1447,13 @@ cleanup3:
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
 		pkgconf_client_set_flags(&pkg_client, want_client_flags | PKGCONF_PKG_PKGF_SKIP_ROOT_VIRTUAL);
-		apply_path(&pkg_client, &world, NULL, maximum_traverse_depth);
+		apply_path(&pkg_client, &world, NULL, 2);
 	}
 
 	if ((want_flags & PKG_VARIABLES) == PKG_VARIABLES)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
-		apply_variables(&pkg_client, &world, NULL, maximum_traverse_depth);
+		apply_variables(&pkg_client, &world, NULL, 2);
 	}
 
 	if (want_variable)
@@ -1438,33 +1461,48 @@ cleanup3:
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
 		pkgconf_client_set_flags(&pkg_client, want_client_flags | PKGCONF_PKG_PKGF_SKIP_ROOT_VIRTUAL);
-		apply_variable(&pkg_client, &world, want_variable, maximum_traverse_depth);
+		apply_variable(&pkg_client, &world, want_variable, 2);
 	}
 
 	if ((want_flags & PKG_REQUIRES) == PKG_REQUIRES)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
-		apply_requires(&pkg_client, &world, NULL, maximum_traverse_depth);
+		apply_requires(&pkg_client, &world, NULL, 2);
 	}
 
 	if ((want_flags & PKG_REQUIRES_PRIVATE) == PKG_REQUIRES_PRIVATE)
 	{
 		want_flags &= ~(PKG_CFLAGS|PKG_LIBS);
 
-		apply_requires_private(&pkg_client, &world, NULL, maximum_traverse_depth);
+		apply_requires_private(&pkg_client, &world, NULL, 2);
 	}
 
 	if ((want_flags & PKG_CFLAGS))
 	{
-		apply_cflags(&pkg_client, &world, NULL, maximum_traverse_depth);
+		apply_cflags(&pkg_client, &world, NULL, 2);
 	}
 
 	if ((want_flags & PKG_LIBS))
 	{
+		if (want_flags & PKG_CFLAGS)
+			printf(" ");
+
 		if (!(want_flags & PKG_STATIC))
+		{
 			pkgconf_client_set_flags(&pkg_client, pkg_client.flags & ~PKGCONF_PKG_PKGF_SEARCH_PRIVATE);
 
-		apply_libs(&pkg_client, &world, NULL, maximum_traverse_depth);
+			/* redo the solution for the library set: free the solution itself, and any cached graph nodes */
+			pkgconf_solution_free(&pkg_client, &world);
+			pkgconf_cache_free(&pkg_client);
+
+			if (!pkgconf_queue_solve(&pkg_client, &pkgq, &world, maximum_traverse_depth))
+			{
+				ret = EXIT_FAILURE;
+				goto out;
+			}
+		}
+
+		apply_libs(&pkg_client, &world, NULL, 2);
 	}
 
 	if (want_flags & (PKG_CFLAGS|PKG_LIBS))
