@@ -67,6 +67,7 @@ typedef struct pkgconf_fragment_ pkgconf_fragment_t;
 typedef struct pkgconf_path_ pkgconf_path_t;
 typedef struct pkgconf_client_ pkgconf_client_t;
 typedef struct pkgconf_cross_personality_ pkgconf_cross_personality_t;
+typedef struct pkgconf_queue_ pkgconf_queue_t;
 
 #define PKGCONF_ARRAY_SIZE(x) (sizeof(x) / sizeof(*(x)))
 
@@ -79,8 +80,13 @@ typedef struct pkgconf_cross_personality_ pkgconf_cross_personality_t;
 #define PKGCONF_FOREACH_LIST_ENTRY_REVERSE(tail, value) \
 	for ((value) = (tail); (value) != NULL; (value) = (value)->prev)
 
-#define LIBPKGCONF_VERSION	10903
-#define LIBPKGCONF_VERSION_STR	"1.9.3"
+#define LIBPKGCONF_VERSION	20001
+#define LIBPKGCONF_VERSION_STR	"2.0.1"
+
+struct pkgconf_queue_ {
+	pkgconf_node_t iter;
+	char *package;
+};
 
 struct pkgconf_fragment_ {
 	pkgconf_node_t iter;
@@ -167,8 +173,7 @@ struct pkgconf_pkg_ {
 	pkgconf_tuple_t *prefix;
 
 	uint64_t serial;
-
-	size_t hits;
+	uint64_t identifier;
 };
 
 typedef bool (*pkgconf_pkg_iteration_func_t)(const pkgconf_pkg_t *pkg, void *data);
@@ -204,6 +209,7 @@ struct pkgconf_client_ {
 	bool already_sent_notice;
 
 	uint64_t serial;
+	uint64_t identifier;
 
 	pkgconf_pkg_t **cache_table;
 	size_t cache_count;
@@ -249,7 +255,7 @@ PKGCONF_API pkgconf_cross_personality_t *pkgconf_cross_personality_default(void)
 PKGCONF_API pkgconf_cross_personality_t *pkgconf_cross_personality_find(const char *triplet);
 PKGCONF_API void pkgconf_cross_personality_deinit(pkgconf_cross_personality_t *personality);
 
-#define PKGCONF_IS_MODULE_SEPARATOR(c) ((c) == ',' || isspace ((unsigned int)(c)))
+#define PKGCONF_IS_MODULE_SEPARATOR(c) ((c) == ',' || isspace ((unsigned char)(c)))
 #define PKGCONF_IS_OPERATOR_CHAR(c) ((c) == '<' || (c) == '>' || (c) == '!' || (c) == '=')
 
 #define PKGCONF_PKG_PKGF_NONE				0x0000
